/*
This file is part of Arapi Extensions

Copyright (c) 2011 Alexis Dorn

Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de

For examples, docs, download and unit tests have a look at 

http://www.alexisdorn.de/arapi-ext

License:

This file is licensed under the terms of
the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
that the code/component(s) do NOT become part of another Open Source or
Commercially licensed development library or toolkit without explicit permission.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL
PROPERTY RIGHTS. See license for more details.

License details: http://www.gnu.org/licenses/lgpl.html

For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
*/
/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */

/**
 * @author Alexis Dorn
 * @class Arapi.app.Application
 * @extends Ext.app.Application
 * 
 * This class provides inheritance to controllers of MVC Application Architecture of Ext JS 4
 * 
 * In complex situations such as multi client software development you might extend existing controllers of your application. 
 * Doing this by using the controllers property of Ext.app.Application you always have to write own getters
 * 
 * such as
 * 
 * 		//Ext.app.Application
 * 
 * 		//Core code	
 * 		getMainController: function (){
 * 			return this.getController("core.Main");
 * 		}
 * 		
 * 		//Client code	
 * 		getMainController: function (){
 * 			return this.getController("client.Main");
 * 		}
 * 
 * 
 * Furthermore the controller property of Ext.app.Application is not inheritable.
 * 
 * 		//Ext.app.Application
 * 
 * 		//Core Code
 * 		controllers: [
 * 			"core.Main",
 * 			"core.User",
 * 			"core.Product",
 * 			"core.Category"
 * 		]
 * 
 * 		//Client Code
 * 		controllers: [
 * 			"client.Main",
 * 			"core.User",
 * 			"core.Product",
 * 			"client.Category"
 * 		]
 * 
 * 
 * Arapi.app.Application enables inheritance of controllers and neutral referencing
 * 
 * 		//Arapi.app.Application
 *  
 * 		//Core Code
 * 		arapiControllers: [
 * 			{
 * 				ref: "Main",
 * 				className: "core.Main"
 * 			},
 * 			{
 * 				ref: "User",
 * 				className: "core.User"
 * 			},
 * 			{
 * 				ref: "Product",
 * 				className: "core.Product"
 * 			},
 * 			{
 * 				ref: "Category",
 * 				className: "core.Category"
 * 			}
 * 		]
 * 
 * 		//Client Code
 * 		arapiControllers: [
 * 			{
 * 				ref: "Main",
 * 				className: "client.Main"
 * 			},
 * 			{
 * 				ref: "Category",
 * 				className: "client.Category"
 * 			}
 * 		]	
 * 		
 * 
 * In both core and client you can access to controllers by using getArapiController
 * 
 *  		//Arapi.app.Application
 * 
 * 			this.getArapiController("Main");
 * 			this.getArapiController("User");
 * 			this.getArapiController("Product");
 * 			this.getArapiController("Category");
 * 
 * 
 * 
 * Start your application with an extension of Arapi.app.Application
 * 
 * 		//Arapi.app.Application
 * 
 * 		Ext.define("MyProject.app.Application",
 * 			extend: "Arapi.app.Application",
 * 			arapiControllers: [
 * 				{
 * 					ref: "Users",
 * 					className: "core.Users"
 * 				},
 * 				(...)
 * 			] 
 * 		)
 * 
 * In app.js you can instantiate your application by
 * 	
 * 		//Arapi.app.Application
 * 
 * 		Ext.application ("MyProject.app.Application", {});		
 * 
 * 
 * This ensures that Sencha SDK builder and creator will work but you have to override the Ext.application method:
 * 
 * 		//Arapi.app.Application
 * 			
 * 		Ext.application = function(config, constFn) {
 * 			var constFn = Ext.isString(constFn) ? constFn : "Ext.app.Application";
 * 			Ext.require(constFn);
 * 			Ext.onReady(function() {
 * 				Ext.create(constFn, config);
 * 			});
 * 		};
 * 
 * 
 * Using Arapi.app.Application you must use arapiControllers instead of controllers property.
 * 
 * 
 * For examples, docs, download and unit tests have a look at <a href="http://www.alexisdorn.de/arapi-ext">http://www.alexisdorn.de/arapi-ext</a>.
 * 
 * Author: Alexis Dorn
 * 
 */

Ext.define('Arapi.app.Application', {
	extend: 'Ext.app.Application',
	onClassExtended: function(cls, data) {
		var extendFn = function (cls, data, last){
			var arapiControllers = data.arapiControllers || [],
				controllers = data.controllers || [],
				superCls = cls.superclass,
				superArapiControllers = superCls.arapiControllers || [],
				superControllers = superCls.controllers || [],
				arapiControllersMixedCollection, 
				i = 0,
				ln,
				last = last || false
			;
			
			//Merge arapiControllers with parent arapiControllers
			arapiControllers = superArapiControllers.concat(arapiControllers);
			
			arapiControllersMixedCollection = new Ext.util.MixedCollection(false, function(item){
				return item.ref;
			});
			
			for (i = 0, ln = arapiControllers.length; i < ln; ++i) {
				arapiControllersMixedCollection.add(arapiControllers[i]);
            }
			
			data.arapiControllers = arapiControllersMixedCollection.getRange();
			
			data.controllers = [];
			data.arapiControllersMixedCollection = arapiControllersMixedCollection;
			for (i = 0, ln = data.arapiControllers.length; i < ln; ++i) {
				data.controllers.push(data.arapiControllers[i]["className"]);
	        }
		};
		
		//Setting onClassExtended
		data.onClassExtended = function (){
			extendFn.apply(this || window, arguments);
		};
		
		extendFn.apply(this || window, [cls, data, true]);
	},
	/**
	 * @cfg {Boolean} debug If debug set to true, application will generate a public namespace for 
	 * instance of application <b>"debug" + this.name</b>
	 * to access application and controllers from outside (e.g. unit tests, performance tests...)
	 */
	debug: false,
	/**
	 * Gets the controller by ref defined in arapiControllers property
	 * @param {String} ref
	 * @return {Arapi.app.Controller/false}
	 */
	getArapiController: function (ref){
		var controller = this.arapiControllersMixedCollection.getByKey(ref);
		if (!Ext.isEmpty(controller)){
			return this.getController(controller.className);
		}
		return false;
	},
	constructor: function (cfg){
		this.callParent([cfg]);
		if (this.debug){
			window["debug" + this.name] = this;
		}
	},
	/**
	 * Returns if application runs in debug mode
	 * @returns {Boolean}
	 */
	isDebug: function (){
		return Ext.isBoolean(this.debug) ? this.debug : false; 
	},
	/**
     * @private
     */
	getModel: function (){
		return this.callParent(arguments);
	},
	/**
     * @private
     */
	getStore: function (){
		return this.callParent(arguments);
	},
	/**
	 * @private
	 */
	getView: function (){
		return this.callParent(arguments);
	},
	/**
	 * @private
	 */
	getController: function (){
		return this.callParent(arguments);
	}
	 /**
     * Inheritable controllers of application.
     * <ul>
     * 	<li><b>ref</b></li>
     * 	<li><b>className</b> in the same way as used in Ext.app.Application controllers definition</li>
     * </ul>
     * @property arapiControllers
     * @type Objects[]
     */
	/**
	 * @hide
     * @cfg {String[]} models  
     */
	/**
	 * @hide
     * @cfg {String[]} stores
     */
	/**
	 * @hide
     * @cfg {String[]} views
     */
	/**
	 * @hide
     * @cfg {String[]} controllers
     */
	/**
	 * @hide
     * @cfg {String[]} refs
     */
});
/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */

/**
 * @author Alexis Dorn
 * @class Arapi.app.Controller
 * @extends Ext.app.Controller
 * 
 * 
 * This class provides inheritance to models, stores, views and refs of MVC Application Architecture of Ext JS 4
 * 
 * In complex situations such as multi client software development you might extend existing models, stores, views and refs of your application. 
 * 
 * Furthermore you need a neutral getter without using the namespaces of core and client. 
 * 
 * In Ext.app.Controller you have to declare your models, stores and views by using an array with string elements. 
 * You cannot extend this properties in extended controllers (client controllers) of your application.
 * 
 * 		//Ext.app.Controller
 * 
 * 		//Core code
 * 		stores: [
 * 			"core.Users",
 * 			"core.Persons",
 * 			"core.Products",
 * 			"core.Category",
 * 		]
 * 
 * 		//Client code
 * 		stores: [
 * 			"client.Users",
 * 			"core.Persons",
 * 			"core.Products",
 * 			"client.Category",
 * 		]
 * 
 * In Ext.app.Controller you can access your stores with
 * 
 * 		//Ext.app.Controller
 * 
 * 		//Core code
 * 		this.getStore("core.Users"); // or this.getCoreUsersStore();
 * 		this.getStore("core.Persons"); // or this.getCorePersonsStore();
 * 		this.getStore("core.Products"); // or this.getCoreProductsStore();
 * 		this.getStore("core.Category"); // or this.getCoreCategoryStore();			
 * 		
 * 		//Client code
 * 		this.getStore("client.Users"); // or this.getClientUsersStore();
 * 		this.getStore("core.Persons"); // or this.getCorePersonsStore();
 * 		this.getStore("core.Products"); // or this.getCoreProductsStore();
 * 		this.getStore("client.Category"); // or this.getClientCategoryStore();
 *
 * So in Ext.app.Controller you need to write some neutral getters to override them in inheritance
 * 		
 *		//Ext.app.Controller
 *
 * 		//Core code
 * 		getUserStore: function (){
 * 			return this.getStore("core.Users");
 * 		}
 * 		(...)
 * 
 * 		//Client code
 * 		getUserStore: function (){
 * 			return this.getStore("client.Users");
 * 		}
 * 		(...)	
 * 
 * Arapi.app.Controller provides inheritance of stores, models, views and refs by using the properties 
 * <ul>
 * 	<li>arapiStores</li>
 * 	<li>arapiModels</li>
 * 	<li>arapiViews</li>
 * 	<li>arapiRefs</li>
 * </ul>
 * 
 * For example:
 *
 * 		//Arapi.app.Controller
 *
 * 		//Core code
 * 		arapiStores: [
 * 			{
 * 				ref: "Users",
 * 				className: "core.Users"
 * 			},
 * 			{
 * 				ref: "Persons",
 * 				className: "core.Persons"
 * 			},
 * 			{
 * 				ref: "Products",
 * 				className: "core.Products"
 * 			},
 * 			{
 * 				ref: "Category",
 * 				className: "core.Category"
 * 			}
 * 		]
 * 
 * 
 * 		//Client code
 * 		arapiStores: [
 * 			{
 * 				ref: "Users",
 * 				className: "client.Users"
 * 			},
 * 			{
 * 				ref: "Category",
 * 				className: "client.Category"
 * 			}
 * 		] 
 * 
 * In both core and client you can access to stores by using getArapiStore
 *
 * 		//Arapi.app.Controller
 *
 * 			this.getArapiStore("Users");
 * 			this.getArapiStore("Persons");
 * 			this.getArapiStore("Products");
 * 			this.getArapiStore("Category");
 * 
 * 
 * Inheritable Models and views will be defined by 
 * 		
 *		//Arapi.app.Controller
 *
 * 		arapiModels: [
 * 			{
 * 				ref: "User",
 * 				className: "core.User"
 * 			}
 * 		],
 * 		arapiViews: [
 * 			{
 * 				ref: "UserWindow",
 * 				className: "core.user.Window"
 * 			}
 * 		]
 * 
 * and you can access them by using the methods in core and in client code
 *
 * 		//Arapi.app.Controller
 *
 * 		this.getArapiModel("User");
 * 
 * 		this.getArapiView("UserWindow");
 * 
 * Arapi.app.Controller also provides inheritance for refs
 * 
 * For example:
 *
 * 		//Arapi.app.Controller
 *
 * 		//Core code
 * 		arapiRefs: [
 * 			{
 * 				ref: "UsersWindow",
 * 				selector: "userwindow",
 * 				autoCreate: true,
 * 				height: 400,
 * 				width: 300,
 * 				modal: true
 * 			},
 * 			{
 * 				ref: "ProductsPanel",
 * 				selector: "viewport > productpanel",
 * 				inheritancePorperty: "this will be overridden in inheritance",
 * 				superProperty: "this should be present in inheritance",
 * 				autoCreate: true
 * 			},
 * 			{
 * 				ref: "CategoryForm",
 * 				selector: "viewport > categorypanel > #catform"
 * 			}
 * 		]
 * 
 * 
 * 		//Client code
 * 		arapiRefs: [
 * 			{
 * 				ref: "ProductsPanel",
 * 				selector: "viewport > clientproductpanel",
 * 				inheritancePorperty: "the client value"
 * 			},
 * 			{
 * 				ref: "CategoryForm",
 * 				selector: "viewport > clientcategorypanel > #clientcatform"
 * 			}
 * 		]
 * 
 * while you can access in both core and client by the getters
 *
 * 		//Arapi.app.Controller
 *		
 * 		this.getUsersWindow();
 * 		this.getProductsPanel();
 * 		this.getCategoryForm();
 * 
 * 
 * Furthermore the configuration properties "inheritancePorperty" and "superProperty" of ProductsPanel arapiRef will still exists in 
 * client extension while "inheritancePorperty" will be overridden with "the client value".
 * 
 * 
 * Arapi.app.Controller provides new or extended features for refs (arapiRefs) to make usage more efficient:
 * 
 * <ul>
 * 	<li><b>selector (extended):</b> In Arapi selector must be a unique component query to avoid getting different components</li>
 * 	<li><b>className (new):</b> In an autoCreate or forceCreate ref you can use className as used in arapiViews instead of using xtype. If className is defined you don't need to define selector property to access the instance. Adventage: Class will be loaded automatically by Ext.Loader</li>
 * 	<li><b>targetSelector (new):</b> In an autoCreate or forceCreate ref you can define an UNIQUE component selector where the new created component will be added (using @link Ext.container.AbstractContainer#add) automatically</li>
 * 	<li><b>nSelector (new):</b> As selector must be unique in Arapi you can use nSelector to get an array of resulting components. Will not work with forceCreate or autoCreate.</li>
 * </ul>
 * 
 * 
 * Using Arapi.app.Application you must use arapiStores, arapiModels, arapiViews and arapiRefs instead of stores, models, views and refs property.
 * 
 * For examples, docs, download and unit tests have a look at <a href="http://www.alexisdorn.de/arapi-ext">http://www.alexisdorn.de/arapi-ext</a>.
 * 
 * Author: Alexis Dorn
 * 
 */

Ext.define("Arapi.app.Controller", {
	extend: "Ext.app.Controller",
	/**
	 * Will be called in class definition pipeline as preprocessor.
	 * Enables inheritance of arapiRefs. 
	 * Enables inheritance of array definition of refs, views, stores and models
	 * Enables defining models, stores and views being object with className and ref
	 * Avoids reference of refs (as in native ext), store, models and views 
	 * @param {Function} cls Subclass
	 * @param {Object} data Data passed into Ext.define
	 * @private
	 */
	onClassExtended: function(cls, data, hooks) {
		var extendFn = function (cls, data){
			var superCls = cls.superclass,
				mixedCollection,
				testCollection,
				testMixedObject,
				addItem,
				i = 0,
				j = 0,
				ln,
				onBeforeClassCreated = (Ext.isEmpty(hooks)) ? data.onBeforeClassCreated : hooks.onBeforeCreated,
				inheritanceItems = ["refs", "models", "stores", "views"],
				superProp = {},
				prop = {},
				item,
				arapiItem,
				beforeCreateFn
			;
			
			//######################################
			//Merge arapiRefs with parent arapiRefs
			for (i = 0; i < inheritanceItems.length; i++){
				arapiItem = "arapi" + Ext.String.capitalize(inheritanceItems[i]);
				item = inheritanceItems[i];
				superProp[arapiItem] = superCls[arapiItem] || [];
				prop[arapiItem] = data[arapiItem] || [];
				
				prop[arapiItem] = superProp[arapiItem].concat(prop[arapiItem] );
				
				mixedCollection = new Ext.util.MixedCollection(false, function(mItem){
					return Ext.String.capitalize(mItem.ref);
				});
				testCollection = {};
				
				for (j = 0, ln = prop[arapiItem].length; j < ln; ++j) {
					testMixedObject = testCollection[prop[arapiItem][j].ref];
					if (!Ext.isEmpty(testMixedObject)){
						addItem = Ext.apply(testMixedObject, prop[arapiItem][j]);
					}else{
						addItem = prop[arapiItem][j];
					}
					mixedCollection.add(Ext.clone(addItem));
					
					testCollection[prop[arapiItem][j].ref] = prop[arapiItem][j];
	            }
				data[arapiItem] = mixedCollection.getRange();
				
				//Setting Date for views, models and stores
				if (arapiItem != "arapiRefs"){
					prop[item] = data[item] || [];
					for (j = 0, ln = data[arapiItem].length; j < ln; ++j) {
						prop[item].push(data[arapiItem][j]["className"]);
		            }
				}
				
			}
			
			//######################################
			//Merge refs, models, stores and views with parent setting
			for (i = 0; i < inheritanceItems.length; i++){
				item = inheritanceItems[i];
				superProp[item] = superCls[item] || [];
				prop[item] = data[item] || [];
				data[item] = Ext.Array.unique(superProp[item].concat(prop[item]));
			}
			
			
			beforeCreateFn = function(cls, data) {
				var module,
					i = 0,
					j = 0,
					item,
					items,
					prefix,
					requires = [],
					modules = [
						{
							prop: "arapiRefs",
							target : "view"
						},
						{
							prop: "arapiViews",
							target : "view"
						},
						{
							prop: "arapiModels",
							target : "model"
						},
						{
							prop: "arapiStores",
							target : "store"
						}
					],
					className = Ext.getClassName(cls),
					namespace = Ext.Loader.getPrefix(className),
					arapiCls = "";
				;
				
				for (i = 0; i < modules.length; i++){
					module = modules[i];
					items = Ext.Array.from(data[module.prop]);
					for (j = 0; j < items.length; j++){
						item = items[j];
						//prefix = Ext.Loader.getPrefix(item);
						if (!Ext.isEmpty(item["className"])){
							arapiCls = namespace + "." + module.target + "." + item["className"];
							requires.push(arapiCls);
							if (module.prop == "arapiRefs"){
								item["arapiViewCls"] = arapiCls;
							}
						}	
					}
				}
				Ext.require(requires, Ext.Function.pass(onBeforeClassCreated, arguments, this));
			};
			if (Ext.isEmpty(hooks)){
				data.onBeforeClassCreated = beforeCreateFn;
			}else{
				hooks.onBeforeCreated = beforeCreateFn;
			}
			
			
		};
		
		//Setting onClassExtended
		data.onClassExtended = function (){
			extendFn.apply(this || window, arguments);
			//return cls.superclass.superclass.$onExtended.apply(this || window, arguments);
		};
		extendFn.apply(this || window, arguments);
	},
	/**
	 * @private
	 * @param cfg
	 */
	constructor: function (cfg){
		this.callParent(arguments);
		if (this.arapiRefs) {
            this.arapiRef(this.arapiRefs);
        }
	},
	/**
	 * Returns same value as this.getView but by ref defined in arapiViews property
	 * @param ref
	 * @return {Function}
	 */
	getArapiView: function (ref){
		return this.getArapiItemsHelper("view", ref);
	},
	/**
	 * Returns same value as this.getStore but by ref defined in arapiStores property
	 * @param ref
	 * @return {Function}
	 */
	getArapiStore: function (ref){
		return this.getArapiItemsHelper("store", ref);
	},
	/**
	 * Returns same value as this.getModel but by ref defined in arapiModels property
	 * @param ref
	 * @return {Function}
	 */
	getArapiModel: function (ref){
		return this.getArapiItemsHelper("model", ref);
	},
	/**
	 * @private
	 * @param getArapiItemsHelper
	 */
	getArapiItemsHelper: function (type, ref){
		var i = 0, 
			items, 
			item, 
			bType = Ext.String.capitalize(type), 
			aType = "arapi" + bType, 
			defProp = aType + "s",
			me = this
		;
		if (Ext.isEmpty(me[aType + "Cache"])){
			me[aType + "Cache"] = {};
		}
		if (Ext.isEmpty(me[aType + "Cache"][ref])){
			items = Ext.Array.from(me[defProp]);
			for (i = 0; i < items.length; i++){
				item = items[i];
				me[aType + "Cache"][item.ref] = item.className;
			}
		}
		if (!Ext.isEmpty(me[aType + "Cache"][ref])){
			return me["get" + bType](me[aType + "Cache"][ref]);
		}
		return me["get" + bType]("");
	},
	/**
	 * @private
	 * @param arapiRefs
	 */
	arapiRef: function (arapiRefs){
		var me = this;
		arapiRefs = Ext.Array.from(arapiRefs);
		Ext.Array.each(arapiRefs, function(info) {
			var ref = info.ref,
				fn = 'get' + Ext.String.capitalize(ref)
			;
			if (!me[fn]){
				me[fn] = Ext.Function.pass(me.arapiGetRef, [ref, info], me);
			}
		});
	},
	/**
	 * Can be defined by className, xtype or selector
	 * className + xtype: autoCreate or forceCreate must be true
	 * className + xtype: targetSelector can be defined to add ref automatically as item. Must be exactly one existing component of selector. Must be a selector for exactly one component
	 * selector: Must be maximal 1 components of selector. Must be a selector for maximal one component. Component must be already instantiated
	 * nSelector: Will return an result of Ext.ComponentQuery.query(nSelector) 
	 * @param ref
	 * @param info
	 * @private
	 */
	arapiGetRef: function (ref, info, cfg){
		this.arapiRefCache = this.arapiRefCache || {};
		var cached = this.arapiRefCache[ref],
			clsName,
			cmp,
			info = info || {},
			cfg = cfg || {}
		;
		info = Ext.apply(info, cfg);
		//The component always has to be instantiated
		if (info.forceCreate){
			//info.className has been defined
			if (Ext.isString(info.arapiViewCls)){
				clsName = info.arapiViewCls;
				cached = Ext.create(clsName, this.arapiMakeRefConfig(info));
			}
			//info.xtype has been defined
			if (Ext.isString(info.xtype)){
				clsName = Ext.ClassManager.getNameByAlias("widget." + info.xtype);
				cached = Ext.create(clsName, this.arapiMakeRefConfig(info));
			}
			//info.targetSelector has been defined
			if (Ext.isString(info.targetSelector)){
				this.arapiAddRef2Target(cached, info.targetSelector);
			}
			return cached;
		}
		if (!cached) {
			if (info.autoCreate){
				//info.className has been defined
				if (!cached && info.autoCreate && Ext.isString(info.arapiViewCls)){
					clsName = info.arapiViewCls;
					this.arapiRefCache[ref] = cached = Ext.create(clsName, this.arapiMakeRefConfig(info));
				}
				//info.xtype has been defined
				if (!cached && info.autoCreate && Ext.isString(info.xtype)){
					clsName = Ext.ClassManager.getNameByAlias("widget." + info.xtype);
					this.arapiRefCache[ref] = cached = Ext.create(clsName, this.arapiMakeRefConfig(info));
				}
				//info.targetSelector has been defined
				if (cached && Ext.isString(info.targetSelector)){
					this.arapiAddRef2Target(cached, info.targetSelector);
				}
			}
			//info.selector has been defined
			if (!cached && Ext.isString(info.selector)){
				cmp = Ext.ComponentQuery.query(info.selector);
				if (cmp.length == 1){
					this.arapiRefCache[ref] = cached = cmp[0];
				}else{
					//TODO: This should be an ArapiError
					Ext.Error.raise({
	                    msg: "More than one or none result found for '" + info.selector + "'"
	                });
				}
			}
			//adding destroy handler to delete caching
			if (cached) {
                cached.on('beforedestroy', function() {
                	this.arapiRefCache.refCache[ref] = null;
                });
            }
			//info.nSelector has been defined
			//there will be no caching of results
			if (!cached && Ext.isString(info.nSelector)){
				cached = Ext.ComponentQuery.query(info.nSelector);
			}
		}
		return cached;
	},
	/**
	 * @param info
	 * @returns {Object}
	 * @private
	 */
	arapiMakeRefConfig: function (info){
		var ownProps = ["arapiViewCls", "autoCreate", "ref", "className", "forceCreate", "xtype", "selector", "targetSelector", "nSelector"],
			cfg = {}
		;
		Ext.Object.each(info, function (key, value){
			if (Ext.Array.indexOf(ownProps, key) == -1){
				cfg[key] = value;
			}
		}, this);
		return cfg;
	},
	/**
	 * 
	 * @param cmp
	 * @param targetSelector
	 * @private
	 */
	arapiAddRef2Target: function (cmp, targetSelector){
		var targetCmp = Ext.ComponentQuery.query(targetSelector);
		if (targetCmp.length == 1){
			targetCmp = targetCmp[0];
			targetCmp.add(cmp);
		}else{
			//TODO: This should be an ArapiError
			Ext.Error.raise({
                msg: "More than one result found for target '" + targetSelector + "'"
            });
		}
	},
	/**
     * @private
     */
	getModel: function (){
		return this.callParent(arguments);
	},
	/**
     * @private
     */
	getStore: function (){
		return this.callParent(arguments);
	},
	/**
	 * @private
	 */
	getView: function (){
		return this.callParent(arguments);
	}
	/**
     * Inheritable views of controller.
     * <ul>
     * 	<li><b>ref</b></li>
     * 	<li><b>className</b> in the same way as used in Ext.app.Controller views definition</li>
     * </ul>  
     * @property arapiViews
     * @type Objects[]
     */
	/**
     * Inheritable models of controller.
     * <ul>
     * 	<li><b>ref</b></li>
     * 	<li><b>className</b> in the same way as used in Ext.app.Controller models definition</li>
     * </ul>  
     * @property arapiModels
     * @type Objects[]
     */
	/**
     * Inheritable stores of controller.
     * <ul>
     * 	<li><b>ref</b></li>
     * 	<li><b>className</b> in the same way as used in Ext.app.Controller stores definition</li>
     * </ul>
     * @property arapiStores
     * @type Objects[]
     */
	/**
     * Inheritable refs of controller.
     * <ul>
     * 	<li><b>ref</b></li>
     * 	<li><b>selector (extended):</b> In Arapi selector must be a unique component query to avoid getting different components</li>
     * 	<li><b>className (new):</b> In an autoCreate or forceCreate ref you can use className as used in arapiViews instead of using xtype. If className is defined you don't need to define selector property to access the instance. . Adventage: Class will be loaded automatically by Ext.Loader</li>
     * 	<li><b>targetSelector (new):</b> In an autoCreate or forceCreate ref you can define an UNIQUE component selector where the new created component will be added (using @link Ext.container.AbstractContainer#add) automatically</li>
     *	<li><b>nSelector (new):</b> As selector must be unique in Arapi you can use nSelector to get an array of resulting components. Will not work with forceCreate or autoCreate.</li>
     *</ul>
     * All other properties will be used as configuration in constructor function if autoCreate or forceCreate is true.  
     * @property arapiRefs
     * @type Objects[]
     */
	/**
     * @cfg models
     * @private
     * @hide 
     */
	/**
     * @cfg stores
     * @private
     * @hide
     */
	/**
     * @cfg views 
     * @private
     * @hide
     */
	/**
     * @cfg refs
     * @private
     * @hide
     */
	
});
/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */
/** 
 * @class Arapi.unittest.JasminePanel
 * @extends Ext.panel.Panel
 * @author Alexis Dorn
 * Tree Panel with Arapi.unittest.plugin.JasmineSuite plugin. 
 * Output of jasmine test results in a tree view.
 * Allows defining the scope of test functions. 
 * With Arapi.unittest.JasminePanel you can run unit tests everywhere inside your application. 
 * 
 * Include jasmine.js from <a href="http://pivotal.github.com/jasmine" target="_blank">http://pivotal.github.com/jasmine</a>
 * 
 * Easy to use API:
 * 
 *     // Be sure that jasmine.js (http://pivotal.github.com/jasmine) 
 *     // is included in your application
 *     var uTestPanel = Ext.create("Arapi.unittest.JasminePanel", {
 *         jasmineCfg: {
 *             fn: function (){
 *                 var me = this;
 *                 describe("Suite 1", function(){
 *                     it("that's ok", function(){
 *                         expect(1 == 1).toBeTruthy();
 *                     });
 *                     it("that's not ok", function(){
 *                         expect(1 != 1).toBeTruthy();
 *                     });
 *                 });
 *             },
 *             scope: this
 *         }
 *     });
 * 		
 *     // myContainer.add(uTestPanel);
 * 		
 *     // Run test with output into tree of jasmine panel
 *     uTestPanel.run();
 * 		
 * 
 * For a tutorial of jasmine have a look at <a href="http://evanhahn.com/?p=181" target="_blank">How do I Jasmine</a> from Evan Hahn.
 * 
 * Tested with jasmine-1.1.0
 */
Ext.define("Arapi.unittest.JasminePanel", {
	requires: [
		"Arapi.unittest.reporter.JasmineTreeReporter",
		"Arapi.unittest.plugin.JasmineSuite",
		"Ext.tip.*"
	],
	xtype: "jasmineunittestpanel",
	extend: "Ext.tree.Panel",
	/**
	 * 
	 * @param cfg
	 */
	constructor: function (cfg){
		var me = this;
		
		if (Ext.isEmpty(cfg.reporterClass)){
			/**
			 * @cfg {String} reporterClass Defaults to 'Arapi.unittest.reporter.JasmineTreeReporter'
			 */
			cfg.reporterClass = "Arapi.unittest.reporter.JasmineTreeReporter";
		}
		
		/**
		 * @cfg {Object} jasmineCfg 
		 * Same configuration used with {@link Arapi.unittest.plugin.JasmineSuite}. For example:
		 * 		jasmineCfg: {
		 *       	title: "Title",
		 *     		fn: function (){describe("Suite 1", {(...)})},
		 * 			scope: this
		 * 		}
		 */
		
		//creating reporter
		this.reporter = Ext.create(cfg.reporterClass, {});
		
		cfg = Ext.apply({}, cfg, {
			jasmineCfg: {},
			//Panel cfg
			rootVisible: true,
			store: this.reporter.getTreeStore(),
			columns: [
				{
					text: "Description",
					dataIndex: "description",
					xtype: 'treecolumn',
					flex: 4
				},
				{
					text: "Passed",
					dataIndex: "passed",
					flex: 1,
					renderer: this.renderPassed
				},
				{
					text: "Error",
					dataIndex: "errorMsg",
					hidden: true
				},
				{
					text: "Stack trace",
					dataIndex: "stackTrace",
					hidden: true
				}
			],
			tbar: [
				{
					text: "Expand All",
					handler: this.expandAll,
					scope: this
				}
			]
		});
		
		//adding suite plugin
		this.addSuitePlugin(cfg.jasmineCfg);
				
		this.callParent([cfg]);
		
		this.addListener ("itemclick", function(view, rec){
	});
		
		this.getView().on('render', function(view) {
		    view.tip = Ext.create('Ext.tip.ToolTip', {
		        target: view.el,
		        delegate: view.itemSelector,
		        trackMouse: true,
		        renderTo: Ext.getBody(),
		        listeners: {
		            beforeshow: function updateTipBody(tip) {
		            	var rec = view.getRecord(tip.triggerElement);
		            	//TODO: could be an suite with errors!!!!
		            	if (Ext.isEmpty(rec.get('errorMsg'))){
		            		tip.update('Results for "' + rec.get('description') + '" => OK');
		            	}else{
		            		tip.update('<b>Errors:</b><br>' + rec.get('errorMsg') + '<br><br><b>Stack trace:</b><br>' + rec.get('stackTrace'));
		            	}
	            		
		            }
		        }
		    });
		});
		
		
	},
	/**
	 * @private
	 */
	addSuitePlugin: function(sCfg){
		this.plugins = this.plugins || [];
		this.plugins.push(Ext.apply({}, sCfg,{
			ptype: "jasminetestsuite",
			reporter: this.reporter
		}));
	},
	/**
	 * Runs jasmine test suite defined in jasmineCfg.fn 
	 */
	run: function (){
		var plugs = this.plugins, i = 0, plug;
		this.reporter.getTreeStore().reset();
		
		for (i = 0; i < plugs.length; i++){
			plug = plugs[i];
			if (plug instanceof Arapi.unittest.plugin.AbstractSuite){
				plug.run(this);
			}
		}
	},
	/**
	 * @private
	 * @param val
	 */
	renderPassed: function (val, obj){
		obj.tdCls = (val == true) ? 'arapi_icon_ok' : 'arapi_icon_error';
		return "";
	}
});
/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */
/** 
 * @class Arapi.unittest.plugin.AbstractSuite
 * @author Alexis Dorn
 */
Ext.define("Arapi.unittest.plugin.AbstractSuite", {
	requires: ["Ext.util.Observable"],
	alias: "plugin.arapiabstracttestsuite",
	mixins: {
		observable: "Ext.util.Observable"
	},
	constructor: function (cfg){
		Ext.apply (this, cfg);
		this.mixins.observable.constructor.apply(this, [cfg]);
	},
	/**
	 * @param {Ext.Component} cmp
	 * @private
	 */
	init: function (cmp){},
	/**
	 * Will execute test
	 * @param {Ext.Component} cmp
	 */
	run: function (cmp){}
});
/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */
/** 
 * @class Arapi.unittest.plugin.JasmineSuite
 * @extends Arapi.unittest.plugin.AbstractSuite
 * @author Alexis Dorn
 * 
 * Plugin for Arapi.unittest.JasminePanel. Manages Jasmine suite and adds reporter to Jasmine.
 */
Ext.define("Arapi.unittest.plugin.JasmineSuite", {
	extend: "Arapi.unittest.plugin.AbstractSuite",
	alias: "plugin.jasminetestsuite",
	constructor: function (cfg){
		/**
		 * @cfg {Arapi.unittest.reporter.JasmineAbstract} reporter Reporter of test
		 */
		cfg = Ext.apply({}, cfg, {
			/**
			 * @cfg {Function} fn Testing function
			 */
			fn: function (){
				it("success: default test of " + this.$className + ". Remove it by defining your own cfg.jasmineFn" ,function(){
					expect(1 == 1).toBeTruthy();
				});
				it("error: default test of " + this.$className + ". Remove it by defining your own cfg.jasmineFn" ,function(){
					expect(1 == 1).not.toBeTruthy();
				});
				describe("Sub suite", function (){
					it("sub success: default test of " + this.$className + ". Remove it by defining your own cfg.jasmineFn" ,function(){
						expect(1 == 1).toBeTruthy();
					});
					it("sub error: default test of " + this.$className + ". Remove it by defining your own cfg.jasmineFn" ,function(){
						expect(1 == 1).not.toBeTruthy();
					});
				});
			},
			/**
			 * @cfg {Object} scope This object in cfg.fn
			 */
			scope: null,
			/**
			 * @cfg {String} title Title of first childnode of root
			 */
			title: "Local Suite (Scope: " + ((!Ext.isEmpty(cfg) && Ext.isObject(cfg) && !Ext.isEmpty(cfg.jasmineScope) && !Ext.isEmpty(cfg.jasmineScope.$className)) ? cfg.jasmineScope.$className : "No cfg.jasmineScope set") + ")"
		});
		
		this.jasmineEnv = jasmine.getEnv();
		this.jasmineEnv.addReporter(cfg.reporter);
		
		this.callParent([cfg]);
	},
	/**
	 * Will execute the suite
	 * @param {Ext.Component} cmp
	 */
	run: function (cmp){
		var me = this;
		//as suite will be marked as finnished after => declare suite always new
		this.suite = describe(this.title, function (){
			if (!Ext.isEmpty(me.scope)){
				me.fn.apply(me.scope);
			}else{
				me.fn();
			}
			
		});
		this.suite.execute();
	}
});
/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */
/** 
 * See Class jasmine.Reporter 
 * <a href="http://pivotal.github.com/jasmine/jsdoc/symbols/jasmine.Reporter.html" target="_blank">http://pivotal.github.com/jasmine/jsdoc/symbols/jasmine.Reporter.html</a> for details.
 * 
 * @class Arapi.unittest.reporter.JasmineAbstract
 * @author Alexis Dorn
 */
Ext.define("Arapi.unittest.reporter.JasmineAbstract", {
	requires: ["Ext.util.Observable"],
	mixins: {
		observable: "Ext.util.Observable"
	},
	constructor: function (cfg){
		cfg = Ext.apply({}, cfg, {});
		Ext.apply (this, cfg);
		this.mixins.observable.constructor.apply(this, [cfg]);
	},
	/**
	 * @param {jasmine.Runner} runner
	 */
	reportRunnerStarting: function (runner){
		
	},
	/**
	 * @param {jasmine.Spec} spec
	 */
	reportSpecStarting: function (spec){
		
	},
	/**
	 * @param {jasmine.Runner} runner
	 */
	reportRunnerResults: function (runner){
		
	},
	/**
	 * @param {jasmine.Suite } suite
	 */
	reportSuiteResults: function (suite){
		
	},
	/**
	 * @param {jasmine.Spec} spec
	 */
	reportSpecResults: function (spec){
		
	},
	/**
	 * @param {String} msg
	 */
	log: function (msg){
		
	}
});

/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */
/** 
 * @class Arapi.unittest.reporter.JasmineTreeModel
 * @extends Ext.data.Model
 * @author Alexis Dorn
 * 
 * Model for Arapi.unittest.reporter.JasmineTreeStore
 */
Ext.define("Arapi.unittest.reporter.JasmineTreeModel", {
    extend: "Ext.data.Model",
    idProperty: "id",
    fields: [
        {
        	name: "id",
        	type: "string"
        },
        {
        	name: "description",
        	type: "string"
        },
        {
        	name: "pId",
        	type: "string"
        },
        {
        	name: "type",
        	type: "string"
        },
        {
        	name: "passed", 
        	type: "boolean"
        },
        {
        	name: "nSpecs", 
        	type: "int"
        },
        {
        	name: "nErrors", 
        	type: "int"
        },
        {
        	name: "errorMsg",
        	type: "string"
        },
        {
        	name: "stackTrace",
        	type: "string"
        }
    ]
});
/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */
/** 
 * @class Arapi.unittest.reporter.JasmineTreeReporter
 * @extends Arapi.unittest.reporter.JasmineAbstract
 * @author Alexis Dorn
 * 
 * Output of Jasmine test results in a Tree Store. Implementation of jasmine.Reporter
 */
Ext.define("Arapi.unittest.reporter.JasmineTreeReporter", {
	extend: "Arapi.unittest.reporter.JasmineAbstract",
	requires: [
		"Arapi.unittest.reporter.JasmineTreeStore"
	],
	constructor: function (cfg){
		cfg = Ext.apply({}, cfg, {
			/**
			 * @cfg {String} storeClass Tree store to save results, default to 'Arapi.unittest.reporter.JasmineTreeStore' 
			 */
			storeClass: "Arapi.unittest.reporter.JasmineTreeStore"
		});
		
		this.store = Ext.create(cfg.storeClass, {});
		this.model = this.store.model;
		this.modelName = this.model.$className;
		
		this.callParent([cfg]);
	},
	/**
	 * Returns the tree store
	 * @return {Arapi.unittest.reporter.JasmineTreeStore}
	 */
	getTreeStore: function (){
		return this.store;
	},
	/**
	 * @param {jasmine.Runner} runner
	 * @private
	 */
	reportRunnerStarting: function (runner){
				
	},
	/**
	 * @param {jasmine.Spec} spec
	 * @private
	 */
	reportSpecStarting: function (spec){
		this.addSuite2Tree(spec.suite);
	},
	/**
	 * @private
	 * @param {jasmine.Suite} suite
	 */
	addSuite2Tree: function (suite){
		var pNode = this.store.getRootNode(), 
			sNode, 
			pId = pNode.getId()
		;
		if (this.store.getNodeById(this.suiteIdent + suite.id)){
			return;
		}
		if (suite.parentSuite) {
			pId = this.suiteIdent + suite.parentSuite.id;
			pNode = this.store.getNodeById(pId);
			if (Ext.isEmpty(pNode)){
				this.addSuite2Tree(suite.parentSuite);
			}
			pNode = this.store.getNodeById(pId);
		}
		sNode = Ext.create(this.modelName, {
			id: this.suiteIdent + suite.id,
			pId: pId, 
			description: suite.description,
			type: "suite",
			passed: false
		});
		pNode.appendChild(sNode);
	},
	/**
	 * @param {jasmine.Runner} runner
	 * @private
	 */
	reportRunnerResults: function (runner){
		var results = runner.results(),
			specs = runner.specs(),
			specCount = 0,
			i = 0,
			message = [],
			sNode = this.store.getRootNode(),
			nText = ""
		;
		for (i = 0; i < specs.length; i++) {
			specCount++;
		}
		message = [
			specCount,
			" spec",
			specCount == 1 ? "" : "s",
			", ",		
			results.failedCount,
			" failure",
			(results.failedCount == 1) ? "" : "s"
		].join("");
		
		nText = "Results: " + message;
		
		sNode.set("passed", results.failedCount == 0);
		sNode.set("description", nText);
		sNode.commit();
	},
	/**
	 * @private
	 */
	reportResult2RootNode: function (specs, errors){
		var message, root = this.store.getRootNode(), rootSpecs;
		root.set("nSpecs", root.get("nSpecs") + specs);
		
		if (errors > 0){
			root.set("passed", false);
		}
		rootSpecs = root.get("nSpecs");
		message = [
			"Results for ",
			rootSpecs,
			" spec",
			rootSpecs == 1 ? "" : "s",
		].join("");
		
		root.set("description", message);
		root.commit();
		
		if (!root.isExpanded() && root.hasChildNodes()){
			root.expand();
		}
	},
	/**
	 * @param {jasmine.Suite} suite
	 * @private
	 */
	reportSuiteResults: function (suite){
		var results = suite.results(),
			sNode = this.store.getNodeById(this.suiteIdent + suite.id),
			specs = suite.specs()
		;
		if (Ext.isEmpty(sNode)){
			return;
		}
		sNode.set("passed", results.passed());
		sNode.set("nSpecs", specs.length);
		sNode.set("nErrors", results.failedCount);
		sNode.commit();
		this.reportResult2RootNode(specs.length, results.failedCount);
	},
	/**
	 * @param {jasmine.Spec} spec
	 * @private
	 */
	reportSpecResults: function (spec){
		var results = spec.results(),
			pNode = this.store.getNodeById(this.suiteIdent + spec.suite.id),
			sNode,
			resultItems,
			result,
			i,
			errorMsg = [],
			stackTrace = []
		; 
		
	    
		resultItems = results.getItems();
		for (i = 0; i < resultItems.length; i++) {
			result = resultItems[i];
			if (result.type == 'expect' && result.passed && !result.passed()) {
				errorMsg.push(result.message);
				if (result.trace.stack){
					stackTrace.push(result.trace.stack);
				}
			}
		}
		
		//TODO: Put this in  this.reportSpecStarting and update only passed 
		sNode = Ext.create(this.modelName, {
			id: this.suiteIdent + spec.suite.id + "-spec-" + spec.id,
			pId: pNode.getId(), 
			description: spec.description,
			type: "spec",
			passed: results.passed(),
			leaf: true,
			errorMsg: errorMsg.join(", "),
			stackTrace: stackTrace.join(", ")
		});
		
		pNode.appendChild(sNode);
	},
	/**
	 * @param {String} msg
	 * @private
	 */
	log: function (msg){
		
	},
	/**
	 * tree store
	 * @private
	 */
	store: false,
	/**
	 * root node
	 * @private
	 */
	root: false,
	/**
	 * model class
	 * @private
	 */
	model: false,
	/**
	 * model class name
	 * @private
	 */
	modelName: false,
	/**
	 * @private
	 */
	suiteIdent: "suite-"
});

/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */
/** 
 * @class Arapi.unittest.reporter.JasmineTreeStore
 * @extends Ext.data.Model
 * @author Alexis Dorn
 * 
 * Tree store for Arapi.unittest.reporter.JasmineTreeReporter
 */
Ext.define("Arapi.unittest.reporter.JasmineTreeStore", {
    extend: "Ext.data.TreeStore",
    requires: [
       	"Arapi.unittest.reporter.JasmineTreeModel"
    ],
    model: "Arapi.unittest.reporter.JasmineTreeModel",
    rootCfg: {
    	expanded: false,
    	id: -1,
    	description: "Test results",
    	nSpecs: 0,
    	nErrors: 0,
    	passed: true
    },
    constructor: function (){
    	//this.root = Ext.apply({}, this.rootCfg);
    	this.callParent(arguments);
    },
    /**
     * Deletes all children under root
     */
    reset: function (){
    	var root = this.getRootNode();
    	if (Ext.isEmpty(root)){
    		this.addRoot();
    		return;
    	}
    	if (root.isExpanded()){
    		root.collapse(false, this.resetAfter, this);
    		return;
    	}
    	this.resetAfter ();
    },
    /**
     * @private
     */
    resetAfter: function (){
    	var root = this.getRootNode(), exRoot, exNode;
    	if (root.hasChildNodes()){
    		root.eachChild(function (node){
    			exNode = node.remove(false);
    			exNode.destroy();
    			exNode = null;
    			delete exNode;
    		}, this);
    	}
    	exRoot = root.remove(true);
    	exRoot.destroy();
    	exRoot = null;
    	delete exRoot;
    	this.addRoot();
    },
    /**
     * @private
     */
    addRoot: function (){
    	this.setRootNode(this.rootCfg);
    }
});

