/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */
/** 
 * Fixes problems with inheritance keeps references to original proxies. 
 * This cannot be fixed in a constructor as suggested in 
 * http://www.sencha.com/forum/showthread.php?141754-4.0.2a-Class-inheritance-keeps-references-to-original-fields/page2
 *	
 * If a model class is used in a store, it will gives the store the referenced proxy and
 * model will not be instantiated. 
 * 
 * Use only proxy configuration as proxy property.
 * 
 * You need to have Ext 4.1 (tested with Ext.4.1.rp1)
 *  
 * @class Arapi.data.Model
 * @extends Ext.data.Model
 * @author Alexis Dorn
 */
Ext.define ("Arapi.data.Model", {
	onClassExtended: function(cls, data) {
		var proxy = Ext.isObject(data.proxy) ? data.proxy : false;
		if (proxy){
			data.superProxyConfig = Ext.clone(proxy);
		}else if (!Ext.isEmpty(cls.superclass.superProxyConfig)){
			data.proxy = cls.superclass.superProxyConfig;
		}
	},
	extend: "Ext.data.Model"
});