/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */
/** 
 * @class Arapi.unittest.plugin.JasmineSuite
 * @extends Arapi.unittest.plugin.AbstractSuite
 * @author Alexis Dorn
 * 
 * Plugin for Arapi.unittest.JasminePanel. Manages Jasmine suite and adds reporter to Jasmine.
 */
Ext.define("Arapi.unittest.plugin.JasmineSuite", {
	extend: "Arapi.unittest.plugin.AbstractSuite",
	alias: "plugin.jasminetestsuite",
	constructor: function (cfg){
		/**
		 * @cfg {Arapi.unittest.reporter.JasmineAbstract} reporter Reporter of test
		 */
		cfg = Ext.apply({}, cfg, {
			/**
			 * @cfg {Function} fn Testing function
			 */
			fn: function (){
				it("success: default test of " + this.$className + ". Remove it by defining your own cfg.jasmineFn" ,function(){
					expect(1 == 1).toBeTruthy();
				});
				it("error: default test of " + this.$className + ". Remove it by defining your own cfg.jasmineFn" ,function(){
					expect(1 == 1).not.toBeTruthy();
				});
				describe("Sub suite", function (){
					it("sub success: default test of " + this.$className + ". Remove it by defining your own cfg.jasmineFn" ,function(){
						expect(1 == 1).toBeTruthy();
					});
					it("sub error: default test of " + this.$className + ". Remove it by defining your own cfg.jasmineFn" ,function(){
						expect(1 == 1).not.toBeTruthy();
					});
				});
			},
			/**
			 * @cfg {Object} scope This object in cfg.fn
			 */
			scope: null,
			/**
			 * @cfg {String} title Title of first childnode of root
			 */
			title: "Local Suite (Scope: " + ((!Ext.isEmpty(cfg) && Ext.isObject(cfg) && !Ext.isEmpty(cfg.jasmineScope) && !Ext.isEmpty(cfg.jasmineScope.$className)) ? cfg.jasmineScope.$className : "No cfg.jasmineScope set") + ")"
		});
		
		this.jasmineEnv = jasmine.getEnv();
		this.jasmineEnv.addReporter(cfg.reporter);
		
		this.callParent([cfg]);
	},
	/**
	 * Will execute the suite
	 * @param {Ext.Component} cmp
	 */
	run: function (cmp){
		var me = this;
		//as suite will be marked as finnished after => declare suite always new
		this.suite = describe(this.title, function (){
			if (!Ext.isEmpty(me.scope)){
				me.fn.apply(me.scope);
			}else{
				me.fn();
			}
			
		});
		this.suite.execute();
	}
});