/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */
/** 
 * @class Arapi.unittest.reporter.JasmineTreeStore
 * @extends Ext.data.Model
 * @author Alexis Dorn
 * 
 * Tree store for Arapi.unittest.reporter.JasmineTreeReporter
 */
Ext.define("Arapi.unittest.reporter.JasmineTreeStore", {
    extend: "Ext.data.TreeStore",
    requires: [
       	"Arapi.unittest.reporter.JasmineTreeModel"
    ],
    model: "Arapi.unittest.reporter.JasmineTreeModel",
    rootCfg: {
    	expanded: false,
    	id: -1,
    	description: "Test results",
    	nSpecs: 0,
    	nErrors: 0,
    	passed: true
    },
    constructor: function (){
    	//this.root = Ext.apply({}, this.rootCfg);
    	this.callParent(arguments);
    },
    /**
     * Deletes all children under root
     */
    reset: function (){
    	var root = this.getRootNode();
    	if (Ext.isEmpty(root)){
    		this.addRoot();
    		return;
    	}
    	if (root.isExpanded()){
    		root.collapse(false, this.resetAfter, this);
    		return;
    	}
    	this.resetAfter ();
    },
    /**
     * @private
     */
    resetAfter: function (){
    	var root = this.getRootNode(), exRoot, exNode;
    	if (root.hasChildNodes()){
    		root.eachChild(function (node){
    			exNode = node.remove(false);
    			exNode.destroy();
    			exNode = null;
    			delete exNode;
    		}, this);
    	}
    	exRoot = root.remove(true);
    	exRoot.destroy();
    	exRoot = null;
    	delete exRoot;
    	this.addRoot();
    },
    /**
     * @private
     */
    addRoot: function (){
    	this.setRootNode(this.rootCfg);
    }
});