/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */
Ext.define('Project.controller.client1.Test', { 
	extend: 'Project.controller.core.Test',
	arapiViews: [
		{
			ref: "Base2Extend",
			className: "client1.dialog.Base2ExtendClient1"
		},
		{
			ref: "OnlyClient1",
			className: "client1.dialog.OnlyClient1"
		}
	],
	unitTestFn: function(){
		var me = this,
			Toolbar = me.application.getArapiController("Toolbar").getToolbar(),
			dynToolbarButton = me.application.getArapiController("Toolbar").getDynButton(),
			productStore = me.application.getArapiController("Products").getArapiStore("Products"),
			productModel = me.application.getArapiController("Products").getArapiModel("Product")
		;
		describe("General Testing", function() {
			it("application is instance of Arapi.app.Application",function(){
				expect(me.application instanceof Arapi.app.Application).toBeTruthy();
		    });
			it("should have correct application name",function(){
		    	expect(me.application.name).toEqual("Project");
		    });
		    it("should have start controller",function(){
		    	expect(me.application.getArapiController("Start")).toBeTruthy();
		    });
		    it("should have toolbar controller",function(){
		    	expect(me.application.getArapiController("Toolbar")).toBeTruthy();
		    });
		    it("should have test controller",function(){
		    	expect(me.application.getArapiController("Test")).toBeTruthy();
		    });
		    it("should have products controller",function(){
		    	expect(me.application.getArapiController("Products")).toBeTruthy();
		    });
		    it("should have reference controller",function(){
		    	expect(me.application.getArapiController("Reference")).toBeTruthy();
		    });
		    it("should have reference sub controller",function(){
		    	expect(me.application.getArapiController("ReferenceSub")).toBeTruthy();
		    });
		});
		
		describe("Client 1 Testing", function() {
			describe("Application has been extended", function() {
				it("application instance of Project.lib.client1.app.Application",function(){
					expect(me.application instanceof Project.lib.client1.app.Application).toBeTruthy();
				});
				it("application extends Project.lib.core.app.Application",function(){
					expect(me.application instanceof Project.lib.core.app.Application).toBeTruthy();
				});
			});
			describe("Controllers has been extended", function() {
			    it("start controller instance of Project.controller.client1.Start",function(){
			    	expect(me.application.getArapiController("Start") instanceof Project.controller.client1.Start).toBeTruthy();
			    });
			    it("start controller extends Project.controller.core.Start",function(){
			    	expect(me.application.getArapiController("Start") instanceof Project.controller.core.Start).toBeTruthy();
			    });
			    it("toolbar controller instance of Project.controller.client1.Toolbar",function(){
			    	expect(me.application.getArapiController("Toolbar") instanceof Project.controller.client1.Toolbar).toBeTruthy();
			    });
			    it("toolbar controller extends Project.controller.core.Toolbar",function(){
			    	expect(me.application.getArapiController("Toolbar") instanceof Project.controller.core.Toolbar).toBeTruthy();
			    });
			    it("products controller instance of Project.controller.client1.Products",function(){
			    	expect(me.application.getArapiController("Products") instanceof Project.controller.client1.Products).toBeTruthy();
			    });
			    it("Products controller extends Project.controller.core.Products",function(){
			    	expect(me.application.getArapiController("Products") instanceof Project.controller.core.Products).toBeTruthy();
			    });
			});
			describe("New controllers has been added", function() {
			    it("client 1 negative controller instance of Project.controller.client1.Negativ",function(){
			    	expect(me.application.getArapiController("Negativ") instanceof Project.controller.client1.Negativ).toBeTruthy();
			    });
			});
		    describe("Account Model has been extended", function() {
			    it("account instance of Project.model.client1.Account",function(){
			    	expect(me.application.getArapiController("Start").getAccount() instanceof Project.model.client1.Account).toBeTruthy();
			    });
			    it("account extends Project.model.core.Account",function(){
			    	expect(me.application.getArapiController("Start").getAccount() instanceof Project.model.core.Account).toBeTruthy();
			    });
			    it("account email is present",function(){
			    	var acc = me.application.getArapiController("Start").getAccount();
			    	expect(acc.get("email")).toBeTruthy();
			    });
			    it("account userId is 1",function(){
			    	var acc = me.application.getArapiController("Start").getAccount();
			    	expect(acc.get("userId")).toEqual(1);
			    });
		    });
		    describe("Products Model and Store has been extended", function() {
		    	it("product store instance of Project.store.client1.Products",function(){
			    	expect(productStore instanceof Project.store.client1.Products).toBeTruthy();
			    });
		    	it("product store extends Project.store.core.Products",function(){
			    	expect(productStore instanceof Project.store.core.Products).toBeTruthy();
			    });
			    it("model of product store is Project.model.client1.Product",function(){
			    	expect(productStore.model.$className).toEqual("Project.model.client1.Product");
			    });
			    it("model of product store extends Project.model.core.Product",function(){
			    	expect(productStore.model.superclass.$className).toEqual("Project.model.core.Product");
			    });
		    });
		    describe("arapiRef of Dynamic Button has been extended", function() {
			    it("dynbutton is in main toolbar",function(){
			    	expect(dynToolbarButton.ownerCt == Toolbar).toBeTruthy();
			    });
			    it("dynbutton core properties are ok",function(){
			    	expect(dynToolbarButton.myCoreProp).toEqual("coreprop");
			    	expect(dynToolbarButton.myCoreObject).toEqual({a: "1", b: "2"});
			    	expect(dynToolbarButton.myCoreArray).toEqual(["1", "2"]);
			    });
			    it("overridden dynbutton over properties are ok",function(){
			    	expect(dynToolbarButton.myOverProp).toEqual("overprop-ext");
			    	expect(dynToolbarButton.myOverObject).toEqual({a: "1-ext", b: "2-ext"});
			    	expect(dynToolbarButton.myOverArray).toEqual(["1-ext", "2-ext"]);
			    });
			    it("dynbutton client 1 properties are ok",function(){
			    	expect(dynToolbarButton.myClient1Prop).toEqual("client1prop");
			    	expect(dynToolbarButton.myClient1Object).toEqual({a: "1", b: "2"});
			    	expect(dynToolbarButton.myClient1Array).toEqual(["1", "2"]);
			    });
			    it("can fire dyn button event, message box is ok",function(){
			    	dynToolbarButton.fireEvent("click", dynToolbarButton);
			    	expect(Ext.Msg.title).toEqual("Dynamic Button");
			    	expect(Ext.Msg.isVisible()).toBeTruthy();
			    	Ext.Msg.close();
			    });
		    });
		    describe("testing more arapiRef and arapiViews", function() {
		    	it("arapiRef with forceCreate is OK",function(){
			    	var dialog1 = me.application.getArapiController("Test").getForceNew();
			    	var dialog2 = me.application.getArapiController("Test").getForceNew();
			    	expect((dialog1 instanceof Ext.window.Window) && (dialog2 instanceof Ext.window.Window)).toBeTruthy();
			    	expect(dialog1 !== dialog2 ).toBeTruthy();
			    	Ext.Msg.close();
			    });
			    it("testing arapiView OnlyCore Dialog",function(){
			    	var dialog = me.application.getArapiController("Test").getArapiView("OnlyCore");
			    	dialog = Ext.create(Ext.ClassManager.getName(dialog), {});
			    	expect(dialog instanceof Project.view.core.dialog.OnlyCore).toBeTruthy();
			    });
			    it("testing arapiView extended Base2Extend Dialog",function(){
			    	var dialog = me.application.getArapiController("Test").getArapiView("Base2Extend");
			    	dialog = Ext.create(Ext.ClassManager.getName(dialog), {});
			    	expect(dialog instanceof Project.view.core.dialog.Base2Extend).toBeTruthy();
			    	expect(dialog instanceof Project.view.client1.dialog.Base2ExtendClient1).toBeTruthy();
			    });	
			    it("testing arapiView OnlyClient1 Dialog",function(){
			    	var dialog = me.application.getArapiController("Test").getArapiView("OnlyClient1");
			    	dialog = Ext.create(Ext.ClassManager.getName(dialog), {});
			    	expect(dialog instanceof Project.view.client1.dialog.OnlyClient1).toBeTruthy();
			    });
			    
		    });
		    describe("Extended grid is present", function() {
		    	var grid;
		    	beforeEach(function (){
		    		grid = me.application.getArapiController("Products").getGrid();
		    	});
		    	it("grid is instanceof Project.view.client1.product.Grid",function(){
			    	expect(grid instanceof Project.view.client1.product.Grid).toBeTruthy();
			    });
		    	it("grid extends Project.view.core.product.Grid",function(){
			    	expect(grid instanceof Project.view.core.product.Grid).toBeTruthy();
			    });
		    	it("grid has new columns category and producer",function(){
		    		var cols = grid.query("gridcolumn"), newCol1 = false, newCol2 = false;
		    		for (var i = 0; i < cols.length; i++){
		    			if (cols[i].dataIndex == "kategorie"){
		    				newCol1 = cols[i];
		    			}
		    			if (cols[i].dataIndex == "hersteller"){
		    				newCol2 = cols[i];
		    			}
		    		}
			    	expect(newCol2 && newCol1).toBeTruthy();
			    });
		    });
		    describe("Negative controll", function() {
		    	var store, rec, storeClient1;
				beforeEach(function (){
					store = me.application.getArapiController("Negativ").getArapiStore("Products");
					rec = store.getRange(0, 1)[0];
					storeClient1 = me.application.getArapiController("Products").getArapiStore("Products");
				});
		    	it("core model account in negative controller still correct",function(){
		    		var accmodel = me.application.getArapiController("Negativ").getArapiModel("Account");
		    		var accTest = accmodel.create({
		    			userId: 4,
		    			firstName: "TEST",
		    			lastName: "TESTNAME",
		    			userName: "test",
		    			email: "test@test.de"
		    		});
		 	    	expect(accTest instanceof Project.model.client1.Account).not.toBeTruthy();
		 	    	expect(accTest instanceof Project.model.core.Account).toBeTruthy();
			    });
		    	it("new arapiRefs DynButton must be own reference",function(){
		    		var bttn = me.application.getArapiController("Negativ").getDynButton();
		    		var bttnNegative = me.application.getArapiController("Toolbar").getDynButton();
		    		expect (bttn.itemId).toEqual("core-toolbar-dynbutton2");
		    		expect (bttn != bttnNegative).toBeTruthy();
		    		expect (bttn.myCoreProp).not.toBeTruthy();
		    	});
		    	it("core product store is not client1 product store",function(){
		    		expect(store instanceof Project.store.client1.Products).not.toBeTruthy();
		 	    	expect(store instanceof Project.store.core.Products).toBeTruthy();
			    });
		    	it("model of product store is Project.model.core.Product",function(){
			    	expect(store.model.$className).toEqual("Project.model.core.Product");
			    });
		    	it("core product store model is not client1 product model",function(){
		    		expect(rec instanceof Project.model.client1.Product).not.toBeTruthy();
		 	    });
		    	it("removing of records in core product store will not affect client 1 product store",function(){
		 	    	store.removeAll();
		 	    	expect(storeClient1.getCount() > 0 && store.getCount() == 0).toBeTruthy();
			    });
		    	
		    });
		    describe("General not extended items of application", function() {
			    it("test controller instance of Project.controller.core.Test",function(){
			    	expect(me.application.getArapiController("Test") instanceof Project.controller.core.Test).toBeTruthy();
			    });
			    it("arapiView Viewport $className is Project.view.core.Viewport",function(){
			    	expect(me.application.getArapiController("Start").getArapiView("Viewport").$className).toEqual("Project.view.core.Viewport");
			    });
			    it("arapiRef getViewport instance of Project.view.core.Viewport",function(){
			    	expect(me.application.getArapiController("Start").getViewport() instanceof Project.view.core.Viewport).toBeTruthy();
			    });
		    });
		    describe("Testing if inheritance does not keep references to superclass", function() {
				var crtlSuper, ctrlSub;
				beforeEach(function (){
					crtlSuper = me.application.getArapiController("Reference");
					crtlSub = me.application.getArapiController("ReferenceSub");
				});
				it("changing arapi ref in sub => test reference",function(){
					crtlSub.arapiRefs[0]["className"] = "mynewvalue";
					expect (crtlSuper.arapiRefs[0]["className"]).not.toEqual("mynewvalue");
			    });
				it("changing arapi view in sub => test reference",function(){
					crtlSub.arapiViews[0]["className"] = "mynewvalue";
					expect (crtlSuper.arapiViews[0]["className"]).not.toEqual("mynewvalue");
			    });
				it("changing arapi store in sub => test reference",function(){
					crtlSub.arapiStores[0]["className"] = "mynewvalue";
					expect (crtlSuper.arapiStores[0]["className"]).not.toEqual("mynewvalue");
			    });
				it("changing arapi model in sub => test reference",function(){
					crtlSub.arapiModels[0]["className"] = "mynewvalue";
					expect (crtlSuper.arapiModels[0]["className"]).not.toEqual("mynewvalue");
			    });
			});
		});
		
	}
});