/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */
Ext.define('Project.controller.core.Start', { 
	extend: 'Arapi.app.Controller',
	arapiModels: [
		{
			ref: "Account",
			className: "core.Account"
		}
	],
	arapiViews: [
		{
			ref: "Viewport",
			className: "core.Viewport"
		}
	],
	arapiRefs: [
		{
			ref: "Viewport",
			selector: "viewport"
		}
	],
	account: false,
	constructor: function (){
		this.addEvents({
			/**
			 * @event LOGIN_OK
			 * Fires when login has been successfull
			 * @param {Project.controller.core.Start} this 
			 */
			LOGIN_OK: true,
			/**
			 * @event VIEWPORT_BUILD
			 * Fires when viewport has been built
			 * @param {Project.controller.core.Start} this 
			 */
			VIEWPORT_BUILD: true
		});
		this.callParent(arguments);
	},
	onLaunch: function (){
		//Loading Account Model
		var aModel = this.getArapiModel("Account");
		aModel.load(1,{ 
			success: this.onAccountSuccess,
			failure: this.onAccountFailure,
			scope: this
		});
	},
	/**
	 * Handles successful account load 
	 * @private
	 * @param {Project.controller.core.model.Account} record 
	 * @param {Ext.data.Operation} operation
	 */
	onAccountSuccess: function (record, operation){
		this.account = record;
		this.fireEvent("LOGIN_OK", this, this.account);
		this.buildViewport();
	},
	/**
	 * @private
	 */
	buildViewport: function (){
		this.getArapiView("Viewport").create({
			listeners: {
				afterrender: {
					fn: this.onStartReady,
					scope: this,
					single: true
				}
			}
		});
	},
	/**
	 * @private
	 * @param cmp
	 */
	onStartReady: function (cmp){
		this.fireEvent("VIEWPORT_BUILD", this, cmp);
	},
	/**
	 * @private
	 */
	onAccountFailure: function (){
		//Doing something on error
		return false;
	},
	/**
	 * @returns {Project.controller.core.model.Account}
	 */
	getAccount: function (){
		return this.account;
	}
});