/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */
Ext.define('Project.controller.core.Test', { 
	extend: 'Project.lib.core.app.Controller',
	arapiRefs: [
		{
			ref: "TestPanel",
			xtype: "jasmineunittestpanel",
			selector: "jasmineunittestpanel",
			autoCreate: true,
			targetSelector: "viewport > #viewport-east"
		},
		{
			ref: "ForceNew",
			className: "core.dialog.ForceNew",
			autoCreate: true,
			forceCreate: true
		}
	],
	arapiViews: [
		{
			ref: "OnlyCore",
			className: "core.dialog.OnlyCore"
		},
		{
			ref: "Base2Extend",
			className: "core.dialog.Base2Extend"
		}
	],
	onLaunch: function (){
		this.getApplication().getArapiController("Toolbar").addListener(
			"BUTTON_1_CLICK", 
			this.runTest, 
			this
		);
	},
	onReady: function (){
		this.getTestPanel({
			jasmineCfg: {
				scope: this,
				fn: this.unitTestFn
			}
		});
	},
	runTest: function (btn){
		this.getTestPanel().run();
	},
	unitTestFn: function(){
		var me = this,
			Toolbar = me.application.getArapiController("Toolbar").getToolbar(),
			dynToolbarButton = me.application.getArapiController("Toolbar").getDynButton(),
			productStore = me.application.getArapiController("Products").getArapiStore("Products"),
			productModel = me.application.getArapiController("Products").getArapiModel("Product")
		;
		describe("General Testing", function() {
			it("application is instance of Arapi.app.Application",function(){
				expect(me.application instanceof Arapi.app.Application).toBeTruthy();
		    });
			it("should have correct application name",function(){
		    	expect(me.application.name).toEqual("Project");
		    });
		    it("should have start controller",function(){
		    	expect(me.application.getArapiController("Start")).toBeTruthy();
		    });
		    it("should have toolbar controller",function(){
		    	expect(me.application.getArapiController("Toolbar")).toBeTruthy();
		    });
		    it("should have test controller",function(){
		    	expect(me.application.getArapiController("Test")).toBeTruthy();
		    });
		    it("should have products controller",function(){
		    	expect(me.application.getArapiController("Products")).toBeTruthy();
		    });
		    it("should have reference controller",function(){
		    	expect(me.application.getArapiController("Reference")).toBeTruthy();
		    });
		    it("should have reference sub controller",function(){
		    	expect(me.application.getArapiController("ReferenceSub")).toBeTruthy();
		    });
		});
		
		describe("Core Testing", function() {

			describe("Testing application and controller basic", function() {
				it("application instance of Project.lib.core.app.Application",function(){
			    	expect(me.application instanceof Project.lib.core.app.Application).toBeTruthy();
			    });
			    it("start controller instance of Project.controller.core.Start",function(){
			    	expect(me.application.getArapiController("Start") instanceof Project.controller.core.Start).toBeTruthy();
			    });
			    it("toolbar controller instance of Project.controller.core.Toolbar",function(){
			    	expect(me.application.getArapiController("Toolbar") instanceof Project.controller.core.Toolbar).toBeTruthy();
			    });
			    it("test controller instance of Project.controller.core.Test",function(){
			    	expect(me.application.getArapiController("Test") instanceof Project.controller.core.Test).toBeTruthy();
			    });

			    it("products controller instance of Project.controller.core.Products",function(){
			    	expect(me.application.getArapiController("Products") instanceof Project.controller.core.Products).toBeTruthy();
			    });
			});
			
			describe("Testing models and stores", function() {
				it("account instance of Project.model.core.Account",function(){
			    	expect(me.application.getArapiController("Start").getAccount() instanceof Project.model.core.Account).toBeTruthy();
			    });
			    it("account email not present",function(){
			    	var acc = me.application.getArapiController("Start").getAccount();
			    	expect(acc.get("email")).not.toBeTruthy();
			    });
			    it("account userId is 1",function(){
			    	var acc = me.application.getArapiController("Start").getAccount();
			    	expect(acc.get("userId")).toEqual(1);
			    });
			    it("product store instance of Project.store.core.Products",function(){
			    	expect(productStore instanceof Project.store.core.Products).toBeTruthy();
			    });
			    it("model of product store is Project.model.core.Product",function(){
			    	expect(productStore.model.$className).toEqual("Project.model.core.Product");
			    });
			});
			
			describe("Testing views and refs", function() {
				it("arapiView Viewport $className is Project.view.core.Viewport",function(){
			    	expect(me.application.getArapiController("Start").getArapiView("Viewport").$className).toEqual("Project.view.core.Viewport");
			    });
			    it("arapiRef getViewport instance of Project.view.core.Viewport",function(){
			    	expect(me.application.getArapiController("Start").getViewport() instanceof Project.view.core.Viewport).toBeTruthy();
			    });
			    
			    it("dynbutton is in main toolbar",function(){
			    	expect(dynToolbarButton.ownerCt == Toolbar).toBeTruthy();
			    });
			    it("dynbutton core properties are ok",function(){
			    	expect(dynToolbarButton.myCoreProp).toEqual("coreprop");
			    	expect(dynToolbarButton.myCoreObject).toEqual({a: "1", b: "2"});
			    	expect(dynToolbarButton.myCoreArray).toEqual(["1", "2"]);
			    });
			    it("dynbutton over properties are ok",function(){
			    	expect(dynToolbarButton.myOverProp).toEqual("overprop");
			    	expect(dynToolbarButton.myOverObject).toEqual({a: "1", b: "2"});
			    	expect(dynToolbarButton.myOverArray).toEqual(["1", "2"]);
			    });
			    it("can fire dyn button event, message box is ok",function(){
			    	dynToolbarButton.fireEvent("click", dynToolbarButton);
			    	expect(Ext.Msg.title).toEqual("Dynamic Button");
			    	expect(Ext.Msg.isVisible()).toBeTruthy();
			    	Ext.Msg.close();
			    });
			    it("arapiRef with forceCreate is OK",function(){
			    	var dialog1 = me.application.getArapiController("Test").getForceNew();
			    	var dialog2 = me.application.getArapiController("Test").getForceNew();
			    	expect((dialog1 instanceof Ext.window.Window) && (dialog2 instanceof Ext.window.Window)).toBeTruthy();
			    	expect(dialog1 !== dialog2 ).toBeTruthy();
			    	Ext.Msg.close();
			    });
			    it("testing arapiView OnlyCore Dialog",function(){
			    	var dialog = me.application.getArapiController("Test").getArapiView("OnlyCore");
			    	dialog = Ext.create(Ext.ClassManager.getName(dialog), {});
			    	expect(dialog instanceof Project.view.core.dialog.OnlyCore).toBeTruthy();
			    });
			    it("testing arapiView Base2Extend Dialog",function(){
			    	var dialog = me.application.getArapiController("Test").getArapiView("Base2Extend");
			    	dialog = Ext.create(Ext.ClassManager.getName(dialog), {});
			    	expect(dialog instanceof Project.view.core.dialog.Base2Extend).toBeTruthy();
			    });
			});
			describe("Testing if inheritance does not keep references to superclass", function() {
				var crtlSuper, ctrlSub;
				beforeEach(function (){
					crtlSuper = me.application.getArapiController("Reference");
					crtlSub = me.application.getArapiController("ReferenceSub");
				});
				it("changing arapi ref in sub => test reference",function(){
					crtlSub.arapiRefs[0]["className"] = "mynewvalue";
					expect (crtlSuper.arapiRefs[0]["className"]).not.toEqual("mynewvalue");
			    });
				it("changing arapi view in sub => test reference",function(){
					crtlSub.arapiViews[0]["className"] = "mynewvalue";
					expect (crtlSuper.arapiViews[0]["className"]).not.toEqual("mynewvalue");
			    });
				it("changing arapi store in sub => test reference",function(){
					crtlSub.arapiStores[0]["className"] = "mynewvalue";
					expect (crtlSuper.arapiStores[0]["className"]).not.toEqual("mynewvalue");
			    });
				it("changing arapi model in sub => test reference",function(){
					crtlSub.arapiModels[0]["className"] = "mynewvalue";
					expect (crtlSuper.arapiModels[0]["className"]).not.toEqual("mynewvalue");
			    });
			});
		});
		
	}
});