/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */
Ext.define('Project.controller.core.Toolbar', { 
	extend: 'Project.lib.core.app.Controller',
	arapiRefs: [
		{
			ref: "Toolbar",
			selector: "viewport > #viewport-north > core-toolbar",
			targetSelector: "viewport > #viewport-north",
			className: "core.Toolbar",
			autoCreate: true
		},
		{
			ref: "DynButton",
			selector: "viewport > #viewport-north > core-toolbar > #core-toolbar-dynbutton",
			targetSelector: "viewport > #viewport-north > core-toolbar",
			autoCreate: true,
			text: "Dynamic Button",
			itemId: "core-toolbar-dynbutton",
			xtype: "button",
			action: "BUTTON_DYN",
			myCoreProp: "coreprop",
			myCoreObject: {
				a: "1",
				b: "2"
			},
			myCoreArray: ["1", "2"],
			myOverProp: "overprop",
			myOverObject: {
				a: "1",
				b: "2"
			},
			myOverArray: ["1", "2"]
		}
	],
	constructor: function (){
		this.addEvents({
			/**
			 * Fires when toolbar is rendered
			 * @event TOOLBAR_READY
			 */
			"TOOLBAR_READY": true
		});
		this.callParent(arguments);
	},
	init: function (){
		this.control({
			"viewport > #viewport-north core-toolbar": {
				afterrender: {
					fn: this.afterRenderToolbar,
					scope: this
				}
			}
		});
	},
	onReady: function (){
		this.getToolbar();
	},
	afterRenderToolbar: function (tbar){
		var i = 0, item;
		
		//Add dynamic button
		this.getDynButton();
		
		//Making this button events 
		tbar.items.each(function (item){
			/**
			 * @event BUTTON_1_CLICK
			 * Fires when button has been clicked
			 * @param {Ext.buton.Button} btn
			 */
			/**
			 * @event BUTTON_DYN_CLICK
			 * Fires when button has been clicked
			 * @param {Ext.buton.Button} btn
			 */
			this.relayEvents(item, ["click"], item.action + "_");
		}, this);
		this.fireEvent("TOOLBAR_READY", this, tbar);
		
		this.addListener("BUTTON_DYN_CLICK", function (){Ext.Msg.alert("Dynamic Button", "Click");});
	},
	/**
	 * see http://www.sencha.com/forum/archive/index.php/t-150572.html?s=a34d1719ef467966837a39894a5c0925
	 * fixed in 4.1-pr1
	 */
	relayEvents : function(origin, events, prefix) {
        prefix = prefix || '';
        var me = this,
            len = events.length,
            i = 0,
            oldName,
            newName;

        for (; i < len; i++) {
            oldName = events[i];
            newName = prefix + oldName;
            me.events[newName] = me.events[newName] || true;
            origin.on(oldName, me.createRelayer(newName));
        }
    }
});