/* 
 * This file is part of Arapi Ext
 *
 * Copyright (c) 2011 Alexis Dorn
 * 
 * Contact:  http://www.alexisdorn.de or info(at)alexisdorn.de
 * 
 * For examples, docs, download and unit tests have a look at 
 * http://www.alexisdorn.de/arapi-ext
 * 
 * License:
 * 
 * This file is licensed under the terms of
 * the Open Source LGPL 3.0 license. Commercial use is permitted to the extent
 * that the code/component(s) do NOT become part of another Open Source or 
 * Commercially licensed development library or toolkit without explicit permission.
 * 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT OF THIRD-PARTY INTELLECTUAL 
 * PROPERTY RIGHTS.  See license for more details.
 * 
 * License details: http://www.gnu.org/licenses/lgpl.html
 * 
 * For use you need a license of Ext JS 4 (see license details under http://www.sencha.com/license.)
 */
/**
 * @class Project.lib.core.app.Application
 * @extends Arapi.app.Application
 * @author Alexis Dorn 
 */
Ext.define("Project.lib.core.app.Application", {
	extend: "Arapi.app.Application",
	requires: [
		"Arapi.unittest.JasminePanel"
	],
	/**
	 * The name of your application. This will also be the namespace for your views, controllers models and stores. 
	 */
	name: "Project",
	/**
	 * The path to the directory which contains all application's classes. 
	 * This path will be registered via Ext.Loader.setPath for the namespace specified in the name config.
	 */
	appFolder: "app",
	/**
	 * Enables inheritance of controllers
	 */
	arapiControllers: [
		{
			ref: "Start",
			className: "core.Start"
		},
		{
			ref: "Toolbar",
			className: "core.Toolbar"
		},
		{
			ref: "Test",
			className: "core.Test"
		},
		{
			ref: "Products",
			className: "core.Products"
		},
		{
			ref: "Reference",
			className: "core.Reference"
		},
		{
			ref: "ReferenceSub",
			className: "core.ReferenceSub"
		}
	],
	/**
	 * True to automatically load and instantiate AppName.view.Viewport before firing the launch function.
	 */
	autoCreateViewport: false,
	/**
	 * @private
	 * @param cfg
	 */
	constructor: function (cfg){
		this.addEvents({
			/**
			 * @event APPLICATION_READY
			 * Fires when application is ready to start
			 * @param {Company.lib.app.Application} this 
			 */
			APPLICATION_READY: true
		});
		
		this.callParent([cfg]);
	},
	/**
	 * Called automatically when the page has completely loaded.
	 * This is an empty function that should be overridden by each application that needs to take action on page load
	 * Will be called before onLaunch method of controllers
	 */
	launch: function(){
		//let's get the controller and add event
		var startController = this.getArapiController("Start");
		startController.addListener ("VIEWPORT_BUILD", this.appReady, this);
	},
	/**
	 * will be called after start controller could get account data successfully
	 * @private
	 * @param {Company.controller.Start} controller
	 * @param {Company.model.Account} account
	 */
	appReady: function (controller, account){
		this.fireEvent("APPLICATION_READY", this);
	}
});